 /*
  * gpmain.c - c-style 'main' function for the GamePark 32
  * Copyright (c)2002 Christian Nowak <chnowak@web.de>
  *
  *
  * This file is part of PlusIV.
  *
  * PlusIV is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * PlusIV is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Foobar; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#define NDEBUG
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "defines.h"
#include "gpgraphic.h"
#include "gpstdio.h"
#include "gpstdlib.h"

#include "archdep/gp32/gpsoundbuf.h"
#include "p4/P4.h"
#include "p4/p4colors.h"
#include "archdep/gp32/kboard.h"

/* External function declarations */
extern void CPU_alignment_cache ( void );
extern void XlatVGABuffer (unsigned char *src, unsigned char *dest);

/* Global variables */
GPDRAWSURFACE gpDraw[2];
GPSOUNDBUF sb;
u8 * screenbuf;
u32 brightness = 0;
int menuselection = 0;

/* Converts 8bit rgb values to a GP32 palette value */
#define GP_RGB24(r,g,b) (((((r>>3))&0x1f)<<11)|((((g>>3))&0x1f)<<6)|((((b>>3))&0x1f)<<1))

/* Sets a single GP32 palette entry */
void GpSetPaletteEntry ( u8 i, u8 r, u8 g, u8 b )
  {
    GP_PALETTEENTRY entry = GP_RGB24(r,g,b);
    GpPaletteEntryChange ( i, 1, &entry, 0 );
  }

/* Set all Plus/4 colors from GP32 palette entry 0 to 127
   alpha is a brightness control (0=black, 16=normal, ...) */
void SetPlus4Palette ( u32 alpha )
  {
    int i;
    for (i=0;i<128;i++)
      {
        u8 c = ((i&0x0f)<<3)|((i>>4)&0x07);
        u32 r = P4Colors[c][0];
        u32 g = P4Colors[c][1];
        u32 b = P4Colors[c][2];

        r = (r*alpha)>>4;
        g = (g*alpha)>>4;
        b = (b*alpha)>>4;
        r = (r>255 ? 255 : r);
        g = (g>255 ? 255 : g);
        b = (b>255 ? 255 : b);
        GpSetPaletteEntry ( i, r, g, b );
      }
  }

/* This function gets called every time
   the sound buffer needs to be refilled */
void mixerfunc ( void * userdata, u8 * stream, int len )
  {
    u16 * buf = (u16*)stream;
    P4 * machine = (P4*)userdata;

    machine->soundbuf = buf;
    P4_RenderSound ( machine );
  }

/* main */
void GpMain (void * arg)
  {
    int * t;
    P4 machine;

    /* Set speed to 132MHz and switch cache on */
    GpClockSpeedChange (132000000, 0x3a011, 3);
    CPU_alignment_cache ();

    /* Allocate memory for the screen rendering buffer */
    screenbuf = calloc(1,320*240);

    /* Initialize the LCD */
    GpGraphicModeSet(8, t);
    GpLcdSurfaceGet(&gpDraw[0], 0);
    GpLcdSurfaceGet(&gpDraw[1], 1);
    GpSurfaceSet(&gpDraw[0]);
    GpLcdEnable();

    /* Initialize the SMC driver and keypad */
    GpFatInit();
    GpKeyInit();

    /* Set the Plus/4 color palette */
    SetPlus4Palette ( 16+brightness );

    /* Clear screen */
    XlatVGABuffer ( screenbuf, gpDraw[0].ptbuffer );
    XlatVGABuffer ( screenbuf, gpDraw[1].ptbuffer );

    /* Set colors for our text and virtual keyboard */
    GpSurfaceFlip(&gpDraw[0]);
    GpSetPaletteEntry ( 0x80, 0,0,0 );        /* Black */
    GpSetPaletteEntry ( 0x81, 255,0,0 );      /* Red */
    GpSetPaletteEntry ( 0x82, 255,255,255 );  /* White */
    GpSetPaletteEntry ( 0x83, 0,255,0 );      /* Green */
    GpSetPaletteEntry ( 0x84, 0,0,255 );      /* Blue */

    /* Initialize the virtual keyboard */
    InitKb(&machine);

    /* Initialize the emulation */
    P4_Init (&machine);
    machine.videoBuf = screenbuf;

    /* Set up our sound stream */
    sb.freq = PCM_M22;
    sb.format = PCM_16BIT;
    sb.samples = machine.soundbufsize;
    sb.userdata = &machine;
    sb.callback = mixerfunc;
    sb.pollfreq = (22050/sb.samples)*2;
    GpPcmInit ( sb.freq, sb.format );
    GpSoundBufStart ( &sb );

    while(1)
      {
        /* Emulate for a single raster line */
        u16 curline = machine.rasterCounter;
        P4_Run(&machine);

        /* Finished rendering the frame ? */
        if (machine.rasterCounter==0 && curline>200)
          {
            /* Copy the buffer to the screen */
            XlatVGABuffer ( screenbuf, gpDraw[0].ptbuffer );
            /* Handle the keypad */
            DoKb(&machine);
          }
      }
  }
